==> ejc-proother-impl-3.0.0-20230714.081952-200-sources.jar: com/ejianc/business/proother/contract/service/impl/ContractBpmServiceImpl.java <==
package com.ejianc.business.proother.contract.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ejianc.business.contractbase.filing.enums.FilingStatusEnum;
import com.ejianc.business.contractbase.pool.enums.ContractPerformanceStateEnum;
import com.ejianc.business.proother.contract.bean.ContractEntity;
import com.ejianc.business.proother.contract.service.IContractAsyncService;
import com.ejianc.business.proother.contract.service.IContractService;
import com.ejianc.business.proother.contract.vo.ContractVO;
import com.ejianc.business.proother.enums.*;
import com.ejianc.business.signaturemanage.api.ISignatureCommonApi;
import com.ejianc.business.signaturemanage.vo.WatermarkVO;
import com.ejianc.foundation.file.api.IAttachmentApi;
import com.ejianc.foundation.file.vo.AttachmentVO;
import com.ejianc.foundation.support.api.IBillTypeApi;
import com.ejianc.foundation.support.api.IParamConfigApi;
import com.ejianc.foundation.support.vo.ParamRegisterSetVO;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.auth.session.UserContext;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.skeleton.billState.service.ICommonBusinessService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Service("contract")
public class ContractBpmServiceImpl implements ICommonBusinessService {

	private Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private IContractService contractService;

	@Autowired
	private IBillTypeApi billTypeApi;

	@Autowired
	private SessionManager sessionManager;

	private static final String WATERMARK_CHECK_PARAM_NAME = "P-00a9W886";
	@Autowired
	private ISignatureCommonApi signatureCommonApi;
	@Autowired
	private IParamConfigApi paramConfigApi;
	@Autowired
	private IContractAsyncService contractAsyncService;
	@Autowired
	private IAttachmentApi attachmentApi;


	/**
	 * 审批节点审批中时节点审批前回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeInApprovalBack(Long billId, Integer state, String billTypeCode, String sign) {
		ContractEntity e = contractService.selectById(billId);
		logger.info("审批节点审批中时节点审批前回调:  当前数据-{}", JSONObject.toJSONString(e));
		//线上起草且文件未同步
		if((DraftTypeEnum.线上起草.getCode().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(e.getDraftType())) && !e.getContractFileSyncFlag()) {
			return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
		}

		return CommonResponse.success("审批处理成功！");
	}

	/**
	 * 有审批流提交完回调
	 *
	 * @param
	 * @return
	 */
	@Override
	public CommonResponse<String> afterSubmitProcessor(Long billId, Integer state, String billTypeCode){
		ContractEntity e = contractService.selectById(billId);

		if(DraftTypeEnum.线上起草.getCode().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(e.getDraftType())) {
			if(null == e.getContractFileId()) {
				//确保线上起草方式时，用户已编辑合同文件
				return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
			}
			if(!e.getContractFileSyncFlag()) {
				return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
			}
		}

		UserContext userContext = sessionManager.getUserContext();
		e.setCommitDate(new Date());
		e.setCommitUserCode(userContext.getUserCode());
		e.setCommitUserName(userContext.getUserName());
		contractService.saveOrUpdate(e,false);

		return CommonResponse.success("合同审批回调处理成功！");
	}

	/**
	 * 终审审核完回调
	 *
	 * @param
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public CommonResponse<String> afterApprovalProcessor(Long billId, Integer state, String billTypeCode) {
		/*
			起草方式为线下签订则不需要签章
			当不使用电子签章时：创建时默认为待签章，单据审批通过后自动更改为【已签章】
			当签章状态变为【已签章】时，合同状态变为【履约中】
		 */
		ContractEntity contractEntity = contractService.selectById(billId);
		if (contractEntity != null){
			if (DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(contractEntity.getDraftType())) {
				if (null == contractEntity.getContractFileId()) {
					//确保线上起草方式时，用户已编辑合同文件
					return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
				}
				if (!contractEntity.getContractFileSyncFlag()) {
					return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
				}
			}
				if((DraftTypeEnum.线下签订.getCode().toString()).equals(contractEntity.getDraftType())) {
				//线下签订合同无需进行签章操作，则审批通过后直接置为已生效状态
				logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
				contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
				contractEntity.setPerformanceStatus(ContractPerformanceStateEnum.履约中.getStateCode());
				//合同已生效，添加生效时间
				contractEntity.setEffectiveDate(new Date());
			} else if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(contractEntity.getDraftType())) {
				if(null == contractEntity.getContractFileId()) {
					//确保线上起草方式时，用户已编辑合同文件
					return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
				}
				if(!contractEntity.getContractFileSyncFlag()) {
					return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
				}
			}
		}

		if(state.equals(BillStateEnum.COMMITED_STATE.getBillStateCode())) {
			//直审时添加单据提交人等信息
			contractEntity.setCommitDate(new Date());
			contractEntity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
			contractEntity.setCommitUserName(sessionManager.getUserContext().getUserName());
		}

		//合同推送合同池
		boolean pushResult = contractService.pushContract(BeanMapper.map(contractEntity, ContractVO.class));
		if(pushResult) {
			contractEntity.setBillPushFlag(BillPushStatusEnum.推送成功.getStatus());
		} else {
			contractEntity.setBillPushFlag(BillPushStatusEnum.未成功推送.getStatus());
		}

		//校验单据存储合同文件Id对应文件是否存在。
		// 若存在则不做任何操作;
		// 若不存在则查询文件中心合同是否存在对应合同附件，若存在则更新到合同单据中;
		// 若文件中也不存在合同对应合同附件，则审批失败
		checkContractFile(contractEntity);

		// 水印系统参数
		CommonResponse<ParamRegisterSetVO> response = paramConfigApi.getByCode(WATERMARK_CHECK_PARAM_NAME);
		if (!response.isSuccess() || response.getData() == null) {
			throw new BusinessException("获取水印系统参数请求失败，失败原因：" + response.getMsg());
		}
		String valueData = response.getData().getValueData();
		Assert.hasText(valueData, "获取的水印系统参数不能为空!");

		// 是否限制： 0:不限制，1:限制
		if ("0".equals(valueData)) {
			// 转换水印参数配置：默认规则
			WatermarkVO watermarkVO = signatureCommonApi.fetchWatermarkConfig(contractEntity.getContractFileId(), contractEntity.getId(),
					contractEntity.getBillCode(), "BT220308000000001", "proOtherContract");
			Assert.notNull(watermarkVO, "获取水印系统参数失败!");
			// 获取上下文并异步调用添加水印
			HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
			HashMap<String, String> headers = new HashMap<>();
			headers.put("authority", request.getHeader("authority"));
			headers.put("ejc-token", request.getHeader("ejc-token"));
			contractAsyncService.fetchWatermarkAttachment(headers, watermarkVO);
		}

		contractService.saveOrUpdate(contractEntity);
		return CommonResponse.success("合同审批回调成功！");
	}

	/**
	 * 有审批流的撤回前回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeHasBpmBack(Long billId, Integer state, String billTypeCode) {
		return bpmBackCheck(billId, state, billTypeCode);
	}

	/**
	 * 弃审前事件回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
		return bpmBackCheck(billId, state, billTypeCode);
	}

	private CommonResponse<String> bpmBackCheck(Long billId, Integer state, String billTypeCode) {
		/*
			弃审逻辑：
			1、审批过程中，支持后台弃审；
			2、当审批通过，且没有走签章流程且没有下游业务时，支持后台弃审；
		 */
		ContractEntity contractEntity = contractService.selectById(billId);

		// 审批可以撤回的签章状态集合
		ArrayList<String> statusList = new ArrayList<>();
		statusList.add(SignatureStatusEnum.待签章.getCode());
		statusList.add(SignatureStatusEnum.已退回.getCode());
		statusList.add(SignatureStatusEnum.已撤回.getCode());
		statusList.add(SignatureStatusEnum.已过期.getCode());

		if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.上传合同.getCode().equals(contractEntity.getDraftType())) {
			if (!statusList.contains(contractEntity.getSignatureStatus())){
				return CommonResponse.error("当前单据已有签章流程，不能撤回！");
			}
		} else if(SignatureStatusEnum.已签章.getCode().equals(contractEntity.getSignatureStatus())) {
			contractEntity.setSignatureStatus(SignatureStatusEnum.待签章.getCode());
		}

		if(ContractPerformanceStateEnum.履约中.getStateCode().equals(contractEntity.getPerformanceStatus())) {
			contractEntity.setPerformanceStatus(ContractPerformanceStateEnum.未签订.getStateCode());
		}

		//合同是否被其他单据引用
		CommonResponse<String> res = billTypeApi.checkQuote(billTypeCode,billId);
		logger.info("平台返回查询被引用情况"+res.isSuccess()+"----"+res.getMsg());

		if(!res.isSuccess()) {
			return CommonResponse.error("当前单据已被下游业务引用，不能弃审！");
		}

		//生效的合同才从合同池做删除操作
		if(BillStateEnum.COMMITED_STATE.getBillStateCode().equals(contractEntity.getBillState()) ||
		BillStateEnum.PASSED_STATE.getBillStateCode().equals(contractEntity.getBillState())) {
			//从合同池删除数据
			String delResult = contractService.delContractFromPool(contractEntity.getId());
			if(StringUtils.isBlank(delResult)) {
				contractEntity.setBillPushFlag(BillPushStatusEnum.未成功推送.getStatus());
			} else {
				logger.error("合同-[{}]弃审，从合同池删除失败!", JSONObject.toJSONString(contractEntity));
				return CommonResponse.error("操作失败，单据从合同池删除失败，原因：" + delResult);
			}

			//若合同属于自动归档的，则将其归档状态改为未归档
			if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.上传合同.equals(contractEntity.getDraftType())) {
				contractEntity.setFilingStatus(FilingStatusEnum.未归档.getTypeCode());
			}
		}

		// 删除文件中心水印文件
		if(contractEntity.getWatermarkContractFileId() != null) {
			boolean delWatermarkflag = contractService.delWatermarkContractFile(contractEntity.getWatermarkContractFileId());
			if (delWatermarkflag) {
				contractEntity.setWatermarkContractFileId(null);
			}
		}

		contractService.saveOrUpdate(contractEntity, false);

		return CommonResponse.success("弃审/撤回前校验回调成功！");
	}

	/**
	 * 检查合同文件
	 *
	 * @param contractEntity
	 */
	private void checkContractFile(ContractEntity contractEntity) {
		//查询合同对饮附件是否存在
		CommonResponse<AttachmentVO> attachResp = attachmentApi.queryDetail(contractEntity.getContractFileId().toString());
		if(!attachResp.isSuccess() || null == attachResp.getData()) {
			logger.info("合同存储文件Id-{}在文件中心不存在，根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息",
					contractEntity.getContractFileId(),
					contractEntity.getId(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile",
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode());
			//合同附件不存在，根据合同Id、业务类型、查询是否存在对应的文件信息，若存在则更新到合同中，否则抛出异常
			CommonResponse<List<AttachmentVO>> attachListResp = attachmentApi.queryListBySourceId(contractEntity.getId(),
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile", "desc");

			logger.info("根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息结果：{}",
					contractEntity.getId(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile",
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode(),
					JSONObject.toJSONString(attachListResp));
			if(attachListResp.isSuccess() || CollectionUtils.isEmpty(attachListResp.getData())) {
				AttachmentVO attachmentVO = attachListResp.getData().get(0);
				logger.info("将合同id-{}对应合同文件Id-{}改为{}", contractEntity.getId(), contractEntity.getContractFileId(), attachmentVO.getId());
				contractEntity.setContractFileId(attachmentVO.getId());
				contractEntity.setContractFilePath(attachmentVO.getTruePath());
			} else {
				throw new BusinessException("审批失败，获取合同文件信息失败, 请检查合同文件！");
			}
		}
	}

}

==> ejc-proother-impl-3.0.0-20240827.101334-237-sources.jar: com/ejianc/business/proother/contract/service/impl/ContractBpmServiceImpl.java <==
package com.ejianc.business.proother.contract.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ejianc.business.contractbase.filing.enums.FilingStatusEnum;
import com.ejianc.business.contractbase.pool.enums.ContractPerformanceStateEnum;
import com.ejianc.business.proother.contract.bean.ContractEntity;
import com.ejianc.business.proother.contract.service.IContractAsyncService;
import com.ejianc.business.proother.contract.service.IContractService;
import com.ejianc.business.proother.contract.vo.ContractVO;
import com.ejianc.business.proother.enums.*;
import com.ejianc.business.signaturemanage.api.ISignatureCommonApi;
import com.ejianc.business.signaturemanage.vo.WatermarkVO;
import com.ejianc.foundation.file.api.IAttachmentApi;
import com.ejianc.foundation.file.vo.AttachmentVO;
import com.ejianc.foundation.support.api.IBillTypeApi;
import com.ejianc.foundation.support.api.IParamConfigApi;
import com.ejianc.foundation.support.vo.ParamRegisterSetVO;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.auth.session.UserContext;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.skeleton.billState.service.ICommonBusinessService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Service("contract")
public class ContractBpmServiceImpl implements ICommonBusinessService {

	private Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private IContractService contractService;

	@Autowired
	private IBillTypeApi billTypeApi;

	@Autowired
	private SessionManager sessionManager;

	private static final String WATERMARK_CHECK_PARAM_NAME = "P-00a9W886";
	@Autowired
	private ISignatureCommonApi signatureCommonApi;
	@Autowired
	private IParamConfigApi paramConfigApi;
	@Autowired
	private IContractAsyncService contractAsyncService;
	@Autowired
	private IAttachmentApi attachmentApi;


	/**
	 * 审批节点审批中时节点审批前回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeInApprovalBack(Long billId, Integer state, String billTypeCode, String sign) {
		ContractEntity e = contractService.selectById(billId);
		logger.info("审批节点审批中时节点审批前回调:  当前数据-{}", JSONObject.toJSONString(e));
		//线上起草且文件未同步
		if((DraftTypeEnum.线上起草.getCode().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(e.getDraftType())) && !e.getContractFileSyncFlag()) {
			return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
		}

		return CommonResponse.success("审批处理成功！");
	}

	/**
	 * 有审批流提交完回调
	 *
	 * @param
	 * @return
	 */
	@Override
	public CommonResponse<String> afterSubmitProcessor(Long billId, Integer state, String billTypeCode){
		ContractEntity e = contractService.selectById(billId);

		if(DraftTypeEnum.线上起草.getCode().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(e.getDraftType())) {
			if(null == e.getContractFileId()) {
				//确保线上起草方式时，用户已编辑合同文件
				return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
			}
			if(!e.getContractFileSyncFlag()) {
				return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
			}
		}

		UserContext userContext = sessionManager.getUserContext();
		e.setCommitDate(new Date());
		e.setCommitUserCode(userContext.getUserCode());
		e.setCommitUserName(userContext.getUserName());
		contractService.saveOrUpdate(e,false);

		return CommonResponse.success("合同审批回调处理成功！");
	}

	/**
	 * 终审审核完回调
	 *
	 * @param
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public CommonResponse<String> afterApprovalProcessor(Long billId, Integer state, String billTypeCode) {
		/*
			起草方式为线下签订则不需要签章
			当不使用电子签章时：创建时默认为待签章，单据审批通过后自动更改为【已签章】
			当签章状态变为【已签章】时，合同状态变为【履约中】
		 */
		ContractEntity contractEntity = contractService.selectById(billId);
		if (contractEntity != null){
			if (DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(contractEntity.getDraftType())) {
				if (null == contractEntity.getContractFileId()) {
					//确保线上起草方式时，用户已编辑合同文件
					return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
				}
				if (!contractEntity.getContractFileSyncFlag()) {
					return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
				}
			}
				if((DraftTypeEnum.线下签订.getCode().toString()).equals(contractEntity.getDraftType())) {
				//线下签订合同无需进行签章操作，则审批通过后直接置为已生效状态
				logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
				contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
				contractEntity.setPerformanceStatus(ContractPerformanceStateEnum.履约中.getStateCode());
				//合同已生效，添加生效时间
				contractEntity.setEffectiveDate(new Date());
			} // 线上起草不使用电子签章，并且已经编辑过合同，审批通过后变为已签章履约中
				else if (DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(contractEntity.getDraftType())
						&& contractEntity.getContractFileId() != null && contractEntity.getContractFileSyncFlag()) {
					//线上起草不使用电子签章合同无需进行签章操作，则审批通过后直接置为已生效状态
					logger.info("合同id-{}为线上起草不使用电子签章直接修改为签章状态并设置为履约中", billId);
					contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
					contractEntity.setPerformanceStatus(PerformanceStatusEnum.履约中.getCode());
					//合同已生效，添加生效时间
					contractEntity.setEffectiveDate(new Date());
				}
//			if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().equals(contractEntity.getDraftType())) {
//				if(null == contractEntity.getContractFileId()) {
//					//确保线上起草方式时，用户已编辑合同文件
//					return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
//				}
//				if(!contractEntity.getContractFileSyncFlag()) {
//					return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
//				}
//			}
		}

		if(state.equals(BillStateEnum.COMMITED_STATE.getBillStateCode())) {
			//直审时添加单据提交人等信息
			contractEntity.setCommitDate(new Date());
			contractEntity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
			contractEntity.setCommitUserName(sessionManager.getUserContext().getUserName());
		}

		//合同推送合同池
		boolean pushResult = contractService.pushContract(BeanMapper.map(contractEntity, ContractVO.class));
		if(pushResult) {
			contractEntity.setBillPushFlag(BillPushStatusEnum.推送成功.getStatus());
		} else {
			contractEntity.setBillPushFlag(BillPushStatusEnum.未成功推送.getStatus());
		}

		//校验单据存储合同文件Id对应文件是否存在。
		// 若存在则不做任何操作;
		// 若不存在则查询文件中心合同是否存在对应合同附件，若存在则更新到合同单据中;
		// 若文件中也不存在合同对应合同附件，则审批失败
		checkContractFile(contractEntity);

		// 水印系统参数
		CommonResponse<ParamRegisterSetVO> response = paramConfigApi.getByCode(WATERMARK_CHECK_PARAM_NAME);
		if (!response.isSuccess() || response.getData() == null) {
			throw new BusinessException("获取水印系统参数请求失败，失败原因：" + response.getMsg());
		}
		String valueData = response.getData().getValueData();
		Assert.hasText(valueData, "获取的水印系统参数不能为空!");

		// 是否限制： 0:不限制，1:限制
		if ("0".equals(valueData)) {
			// 转换水印参数配置：默认规则
			WatermarkVO watermarkVO = signatureCommonApi.fetchWatermarkConfig(contractEntity.getContractFileId(), contractEntity.getId(),
					contractEntity.getBillCode(), "BT220308000000001", "proOtherContract");
			Assert.notNull(watermarkVO, "获取水印系统参数失败!");
			// 获取上下文并异步调用添加水印
			HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
			HashMap<String, String> headers = new HashMap<>();
			headers.put("authority", request.getHeader("authority"));
			headers.put("ejc-token", request.getHeader("ejc-token"));
			contractAsyncService.fetchWatermarkAttachment(headers, watermarkVO);
		}

		contractService.saveOrUpdate(contractEntity);
		return CommonResponse.success("合同审批回调成功！");
	}

	/**
	 * 有审批流的撤回前回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeHasBpmBack(Long billId, Integer state, String billTypeCode) {
		return bpmBackCheck(billId, state, billTypeCode);
	}

	/**
	 * 弃审前事件回调
	 *
	 * @param billId
	 * @param state
	 * @return
	 */
	@Override
	public CommonResponse<String> beforeAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
		return bpmBackCheck(billId, state, billTypeCode);
	}

	private CommonResponse<String> bpmBackCheck(Long billId, Integer state, String billTypeCode) {
		/*
			弃审逻辑：
			1、审批过程中，支持后台弃审；
			2、当审批通过，且没有走签章流程且没有下游业务时，支持后台弃审；
		 */
		ContractEntity contractEntity = contractService.selectById(billId);

		// 审批可以撤回的签章状态集合
		ArrayList<String> statusList = new ArrayList<>();
		statusList.add(SignatureStatusEnum.待签章.getCode());
		statusList.add(SignatureStatusEnum.已退回.getCode());
		statusList.add(SignatureStatusEnum.已撤回.getCode());
		statusList.add(SignatureStatusEnum.已过期.getCode());

		if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.上传合同.getCode().equals(contractEntity.getDraftType())) {
			if (!statusList.contains(contractEntity.getSignatureStatus())){
				return CommonResponse.error("当前单据已有签章流程，不能撤回！");
			}
		} else if(SignatureStatusEnum.已签章.getCode().equals(contractEntity.getSignatureStatus())) {
			contractEntity.setSignatureStatus(SignatureStatusEnum.待签章.getCode());
		}

		if(ContractPerformanceStateEnum.履约中.getStateCode().equals(contractEntity.getPerformanceStatus())) {
			contractEntity.setPerformanceStatus(ContractPerformanceStateEnum.未签订.getStateCode());
		}

		//合同是否被其他单据引用
		CommonResponse<String> res = billTypeApi.checkQuote(billTypeCode,billId);
		logger.info("平台返回查询被引用情况"+res.isSuccess()+"----"+res.getMsg());

		if(!res.isSuccess()) {
			return CommonResponse.error("当前单据已被下游业务引用，不能弃审！");
		}

		//生效的合同才从合同池做删除操作
		if(BillStateEnum.COMMITED_STATE.getBillStateCode().equals(contractEntity.getBillState()) ||
		BillStateEnum.PASSED_STATE.getBillStateCode().equals(contractEntity.getBillState())) {
			//从合同池删除数据
			String delResult = contractService.delContractFromPool(contractEntity.getId());
			if(StringUtils.isBlank(delResult)) {
				contractEntity.setBillPushFlag(BillPushStatusEnum.未成功推送.getStatus());
			} else {
				logger.error("合同-[{}]弃审，从合同池删除失败!", JSONObject.toJSONString(contractEntity));
				return CommonResponse.error("操作失败，单据从合同池删除失败，原因：" + delResult);
			}

			//若合同属于自动归档的，则将其归档状态改为未归档
			if(DraftTypeEnum.线上起草.getCode().equals(contractEntity.getDraftType()) || DraftTypeEnum.上传合同.equals(contractEntity.getDraftType())) {
				contractEntity.setFilingStatus(FilingStatusEnum.未归档.getTypeCode());
			}
		}

		// 删除文件中心水印文件
		if(contractEntity.getWatermarkContractFileId() != null) {
			boolean delWatermarkflag = contractService.delWatermarkContractFile(contractEntity.getWatermarkContractFileId());
			if (delWatermarkflag) {
				contractEntity.setWatermarkContractFileId(null);
			}
		}

		contractService.saveOrUpdate(contractEntity, false);

		return CommonResponse.success("弃审/撤回前校验回调成功！");
	}

	/**
	 * 检查合同文件
	 *
	 * @param contractEntity
	 */
	private void checkContractFile(ContractEntity contractEntity) {
		//查询合同对饮附件是否存在
		CommonResponse<AttachmentVO> attachResp = attachmentApi.queryDetail(contractEntity.getContractFileId().toString());
		if(!attachResp.isSuccess() || null == attachResp.getData()) {
			logger.info("合同存储文件Id-{}在文件中心不存在，根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息",
					contractEntity.getContractFileId(),
					contractEntity.getId(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile",
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode());
			//合同附件不存在，根据合同Id、业务类型、查询是否存在对应的文件信息，若存在则更新到合同中，否则抛出异常
			CommonResponse<List<AttachmentVO>> attachListResp = attachmentApi.queryListBySourceId(contractEntity.getId(),
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile", "desc");

			logger.info("根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息结果：{}",
					contractEntity.getId(),
					("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "subContractFile" : "contractFile",
					ProotherBillTypeEnum.其他支出合同.getBillTypeCode(),
					JSONObject.toJSONString(attachListResp));
			if(attachListResp.isSuccess() || CollectionUtils.isEmpty(attachListResp.getData())) {
				AttachmentVO attachmentVO = attachListResp.getData().get(0);
				logger.info("将合同id-{}对应合同文件Id-{}改为{}", contractEntity.getId(), contractEntity.getContractFileId(), attachmentVO.getId());
				contractEntity.setContractFileId(attachmentVO.getId());
				contractEntity.setContractFilePath(attachmentVO.getTruePath());
			} else {
				throw new BusinessException("审批失败，获取合同文件信息失败, 请检查合同文件！");
			}
		}
	}

}
